//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
//   #define B_DEVELOP
   #include "btrace.hpp"
#endif

#ifndef _PHYSMEM_
   #include "physmem.hpp"
#endif

#include <cstring>

#ifndef _STDEXCEPT_
   #include <stdexcept>
#endif

#ifndef _OSIFACE_
   #include "osiface.hpp"
#endif

using namespace std;
using namespace Uniio;

//-----------------------------------------------------------------------------
PhysicalMemory::PhysicalMemory ( const unsigned long inPhysicalAddress
                               , const unsigned long inSize
                               , const bool fWriteAccess
                               )
   : theSize ( inSize )
   , theWriteAccess (fWriteAccess)
{
BFUNCTRACE_DEVELOP ();

thePtr = OSInterface::osinterface()
                     .mapMemory ( inPhysicalAddress
                                , inSize
                                , fWriteAccess
                                );
} // end constructor 


//-----------------------------------------------------------------------------
PhysicalMemory::~PhysicalMemory ()
{
BFUNCTRACE_DEVELOP ();

OSInterface::osinterface().unmapMemory ( thePtr );
} // end destructor


//-----------------------------------------------------------------------------
PhysicalMemory &
PhysicalMemory::writeMemoryBlock ( const void * const inDataToWrite
                                 , const unsigned long inBlockSize
                                 , const unsigned long inStartOffset
                                 )
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (theWriteAccess, "Write access is not enabled on mapped physical memory");
BASSERTTEXT (inBlockSize+inStartOffset <= theSize, "Memory block to write exceeds boundaries of mapped physical memory");

memcpy ( ((char*)thePtr)+inStartOffset
       , inDataToWrite
       , inBlockSize );

return * this;
} // end PhysicalMemory::writeMemoryBlock (..)


//-----------------------------------------------------------------------------
const PhysicalMemory &
PhysicalMemory::readMemoryBlock ( void * inBuffer
                                , const unsigned long inBlockSize
                                , const unsigned long inStartOffset
                                ) const
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (inBlockSize+inStartOffset <= theSize, "Memory block to read exceeds boundaries of mapped physical memory");

memcpy ( inBuffer
       , ((char*)thePtr)+inStartOffset
       , inBlockSize );

return * this;
} // end PhysicalMemory::readMemoryBlock (..)

//-----------------------------------------------------------------------------
long PhysicalMemory::readLong  (const unsigned long inStartOffset) const
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (sizeof(long)+inStartOffset <= theSize, "Start offset to read exceeds boundaries of mapped physical memory");
return *(long*)(((char*)thePtr)+inStartOffset);
}

//-----------------------------------------------------------------------------
short PhysicalMemory::readShort (const unsigned long inStartOffset) const
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (sizeof(short)+inStartOffset <= theSize, "Start offset to read exceeds boundaries of mapped physical memory");
return *(short*)(((char*)thePtr)+inStartOffset);
}

//-----------------------------------------------------------------------------
char PhysicalMemory::readChar  (const unsigned long inStartOffset) const
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (sizeof(char)+inStartOffset <= theSize, "Start offset to read exceeds boundaries of mapped physical memory");
return *(((char*)thePtr)+inStartOffset);
}

//-----------------------------------------------------------------------------
PhysicalMemory & PhysicalMemory::writeLong  ( const unsigned long inStartOffset
                               , const long inData
                               )
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (theWriteAccess, "Write access is not enabled on mapped physical memory");
BASSERTTEXT (sizeof(long)+inStartOffset <= theSize, "Start offset to write exceeds boundaries of mapped physical memory");

*(long*)(((char*)thePtr)+inStartOffset) = inData;
return *this;
}

//-----------------------------------------------------------------------------
PhysicalMemory & PhysicalMemory::writeShort ( const unsigned long inStartOffset
                               , const short inData
                               )
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (theWriteAccess, "Write access is not enabled on mapped physical memory");
BASSERTTEXT (sizeof(short)+inStartOffset <= theSize, "Start offset to write exceeds boundaries of mapped physical memory");

*(short*)(((char*)thePtr)+inStartOffset) = inData;
return *this;
}

//-----------------------------------------------------------------------------
PhysicalMemory & PhysicalMemory::writeChar  ( const unsigned long inStartOffset
                               , const char inData
                               )
{
BFUNCTRACE_DEVELOP ();
BASSERTTEXT (theWriteAccess, "Write access is not enabled on mapped physical memory");
BASSERTTEXT (sizeof(char)+inStartOffset <= theSize, "Start offset to write exceeds boundaries of mapped physical memory");

*(((char*)thePtr)+inStartOffset) = inData;
return *this;
}
